package net.knarcraft.blockhunt.manager;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.MemoryStorage;
import net.knarcraft.blockhunt.config.ConfigKey;
import net.knarcraft.blockhunt.config.Key;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.Objects;
import java.util.logging.Level;

public class MessageManager {

    /**
     * Send a message to a player. Also replaces the "%player%" variable to the
     * player's name.
     *
     * @param player  The player receiving the message.
     * @param message Message which needs to be sent to the player.
     * @param vars    Variables. Separated with a - . Ex: "playerName-" +
     *                player.getName();
     */
    public static void sendMessage(Player player, String message, String... vars) {
        if (player == null) {
            Bukkit.getConsoleSender().sendMessage(MessageManager.replaceAll(message.replaceAll("%player%", "Console"), vars));
        } else {
            player.sendMessage(MessageManager.replaceAll(message.replaceAll("%player%", player.getDisplayName()), vars));
        }
    }

    /**
     * Send a message to a player from a Config. Also replaces the "%player%"
     * variable to the player's name.
     *
     * @param player   The player receiving the message.
     * @param location Location in the config of the message being sent.
     * @param vars     Variables. Separated with a - . Ex: "playerName-" +
     *                 player.getName();
     */
    public static void sendFMessage(Player player, Key location, String... vars) {
        Object value = location.getConfigManager().getFile().get(location.getPath());
        if (value == null) {
            BlockHunt.plugin.getLogger().log(Level.SEVERE, "Unable to read value for " + location.getPath());
            Objects.requireNonNullElseGet(player, Bukkit::getConsoleSender).sendMessage("An error occurred while " +
                    "generating a BlockHunt message. Please notify an administrator.");
            return;
        }
        String playerString = player == null ? "Console" : player.getDisplayName();
        String message = MessageManager.replaceAll(value.toString().replaceAll("%player%", playerString), vars);
        Objects.requireNonNullElseGet(player, Bukkit::getConsoleSender).sendMessage(message);
    }

    /**
     * Send a message to all players online. Also replaces the "%player%"
     * variable to the player's name.
     *
     * @param message Message which needs to be sent to the player.
     * @param vars    Variables. Seperated with a - . Ex: "playerName-" +
     *                player.getName();
     */
    public static void broadcastMessage(String message, String... vars) {
        for (Player player : Bukkit.getOnlinePlayers()) {
            player.sendMessage(MessageManager.replaceAll(message.replaceAll("%player%", player.getDisplayName()), vars));
        }
        Bukkit.getConsoleSender().sendMessage(MessageManager.replaceAll(message.replaceAll("%player%", "Console"), vars));
    }

    /**
     * Send a message to all players online from a Config. Also replaces the
     * "%player%" variable to the player's name.
     *
     * @param location Location in the config of the message being sent.
     * @param vars     Variables . Separated with a - . Ex: "playerName-" +
     *                 player.getName();
     */
    public static void broadcastFMessage(Key location, String... vars) {
        for (Player player : Bukkit.getOnlinePlayers()) {

            player.sendMessage(MessageManager.replaceAll(location.getConfigManager().getFile().get(
                    location.getPath()).toString().replaceAll("%player%", player.getDisplayName()), vars));
        }
        Bukkit.getConsoleSender().sendMessage(MessageManager.replaceAll(location.getConfigManager().getFile().get(
                location.getPath()).toString().replaceAll("%player%", "Console"), vars));
    }

    /**
     * Replace all variables.
     *
     * @param message Message which needs to be replaced.
     * @param vars    Variables. Seperated with a - . Ex: "playerName-" +
     *                player.getName();
     * @return String with the message with values replaced
     */
    public static String replaceAll(String message, String... vars) {
        return MessageManager.replaceColours(MessageManager.replaceColourVars(MessageManager.replaceVars(message, vars)));
    }

    /**
     * Replace default Minecraft colour codes.
     *
     * @param message Message which needs to be replaced.
     * @return Colour replaced message.
     */
    public static String replaceColours(String message) {
        return message.replaceAll("(&([a-fk-or0-9]))", "\u00A7$2");
    }

    /**
     * Replace colour codes of this plugin.
     *
     * @param message Message which needs to be replaced.
     * @return Colour replaced message.
     */
    public static String replaceColourVars(String message) {
        message = message.replaceAll("%N", CType.NORMAL());
        message = message.replaceAll("%W", CType.WARNING());
        message = message.replaceAll("%E", CType.ERROR());
        message = message.replaceAll("%A", CType.ARG());
        message = message.replaceAll("%H", CType.HEADER());
        message = message.replaceAll("%TAG", CType.TAG());
        return message;
    }

    /**
     * Replace custom variables.
     *
     * @param message Message which needs to be replaced.
     * @param vars    Variables. Seperated with a - . Ex: "playerName-" +
     *                player.getName();
     * @return Replaced String.
     */
    public static String replaceVars(String message, String... vars) {
        for (String var : vars) {
            String[] split = var.split("-");
            message = message.replaceAll("%" + split[0] + "%", split[1]);
        }
        return message;
    }

    public static class CType {

        public static String NORMAL() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_NORMAL);
        }

        public static String WARNING() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_WARNING);
        }

        public static String ERROR() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_ERROR);
        }

        public static String ARG() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_ARGUMENT);
        }

        public static String HEADER() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_HEADER);
        }

        public static String TAG() {
            return (String) MemoryStorage.config.get(ConfigKey.CHAT_HEADER) + MemoryStorage.config.get(ConfigKey.CHAT_TAG) + MemoryStorage.config.get(ConfigKey.CHAT_NORMAL);
        }
    }
}
